#include "global.h"

#include "KeyboardHandler.hpp"
#include "MouseHandler.hpp"
#include "VirtualJoystick.hpp"

#include "Game.hpp"

#include "Frame.hpp"
#include "Music.h"

volatile static int Tick = 0;
static void DoTick () { 
	Tick++;
}
END_OF_FUNCTION(DoTick)

Log gLog;


Game::Game(int maxfps) {

	gLog.start("gamelog.txt", Log::vDebug);

	Tick = 0;
	frame = 0;
	fps = maxfps;
	refresh = true;

	// Install Allegro timer to have constant fps
	LOCK_VARIABLE(Tick);
    LOCK_FUNCTION(DoTick);
	install_int_ex(DoTick, BPS_TO_TIMER(fps));


	KeyboardHandler *kh = new KeyboardHandler();
	MouseHandler *mh = new MouseHandler();
	inputManager.add(kh);
	inputManager.add(mh);
	
	joystick = new VirtualJoystick(inputManager);
	
	joystick->addInput(VirtualJoystick::UP, kh->newInput("W"));
	joystick->addInput(VirtualJoystick::DOWN, kh->newInput("S"));
	joystick->addInput(VirtualJoystick::LEFT, kh->newInput("A"));
	joystick->addInput(VirtualJoystick::RIGHT, kh->newInput("D"));
	
	joystick->addInput(VirtualJoystick::A, mh->newInput("left"));
	joystick->addInput(VirtualJoystick::B, mh->newInput("right"));
	joystick->addInput(VirtualJoystick::C, kh->newInput("Left Ctrl"));



	mp = new MusicPlayer;
	mp->add(rm.load<Music>("data/spook.xm"));
	//mp->playNext();

}

Game::~Game() {
	delete joystick;
	delete mp;
	remove_int(DoTick);
}

void Game::run() {
	running = true;
	int rendTick = 1;
	
	while (running) {
		if (Tick > 30) Tick = 30;
		while (Tick > 0 && running) {
			joystick->poll();
			mp->update();
			frame->update();
			refresh = true;
			Tick--;
		}
		if (refresh && running) {
			frame->draw();
			refresh = false;
			rendTick = Tick;
		}
		if (Tick <= 0) {
			rest(1);
		}
	}
}
void Game::setFrame(Frame *_frame)  {
	if (frame)
		delete frame;
	frame = _frame;
	frame->setGame(this);
}
void Game::stop() {
	running = false;
}
