
#include "GameFrame.hpp"
#include "Game.hpp"
#include "VirtualJoystick.hpp"
#include "Input.hpp"
#include "Player.hpp"
#include "OLBitmap.hpp"
#include "Math.hpp"
#include "Particle.hpp"


Player::Player(GameFrame *_frame) : frame(_frame) {
	yshape.Add(Vec2D(0.0f,-64.0f));
	yshape.Add(Vec2D(15.0f,0.0f));
	yshape.Add(Vec2D(-15.0f,0.0f));
	
	xshape.Add(Vec2D(-32.0f,-64.0f));
	xshape.Add(Vec2D(32.0f,-64.0f));
	xshape.Add(Vec2D(32.0f,-30.0f));
	xshape.Add(Vec2D(-32.0f,-30.0f));
	
	
	pos.x = 300;
	pos.y = 200;
	hurt = 0;
	arm = frame->game->rm.load<OLBitmap>("data/arm.png")->b;
	run.addSeries(frame->game->rm, "data/3vil/run",1,".png");
	run.setRate(0.3);
	stand.addSeries(frame->game->rm, "data/3vil/standing",1,".png");
	stand.setRate(0.3);
	jump.addSeries(frame->game->rm, "data/3vil/jumping",1,".png");
	jump.setRate(0.3);
	hurting.addSeries(frame->game->rm, "data/3vil/hurt",1,".png");
	hurting.setRate(0.1);
	ascii.addSeries(frame->game->rm, "data/ascii/ascii",1,".png");
	anim = &run;
	flip = false;
	angle = 0;
	offset.x = 64;
	offset.y = 110;
	armpos.x = 10;
	armpos.y = -60;
	onground = false;
	onside = 0;
	charge = 0;
	rate = 0;
	jumpcount = 0;
	health = 100;
	power = 100;
}
Player::~Player() {
}

void Player::update(float timerate) {
	if (health<=0)
		hurt = 100;
	if (hurt>0) {
		hurt-=timerate;
		timerate *= 0.5;
	}
	delta.y += 0.2*timerate;
	Vec2D newpos = pos+delta*timerate;
	if (onground) {
		newpos.x -= timerate*(groundtype==2? 10.0+frame->trainspeed : frame->trainspeed);
		if (groundtype==2)
			damage(5);
	}
	onground = false;
	groundtype = 0;
	onside = 0;
	if (health>0) {
		// In danger already?
		Collision c = frame->level.getCollision(xshape,Placement(pos));
		if (c.IsCollision()) {
			bool ok = false;
			for (int i=0; i<15; i++) {
				pos.x++;
				c = frame->level.getCollision(xshape,Placement(pos));
				if (!c.IsCollision()) {
					ok = true;
					break;
				}
			}
			if (!ok) {
				pos.x-=3;
				for (int i=0; i<15; i++) {
					pos.x--;
					c = frame->level.getCollision(xshape,Placement(pos));
					if (!c.IsCollision()) {
						ok = true;
						break;
					}
				}
			}
			if (!ok) {
				pos.x+=3;
				hurt = 20;
				// SQUASHED X
			}
		}
		// In danger already?
		c = frame->level.getCollision(yshape,Placement(pos));
		if (c.IsCollision()) {
			bool ok = false;
			for (int i=0; i<3; i++) {
				pos.y--;
				c = frame->level.getCollision(yshape,Placement(pos));
				if (!c.IsCollision()) {
					ok = true;
					break;
				}
			}
			if (!ok) {
				pos.y+=3;
				for (int i=0; i<3; i++) {
					pos.y++;
					c = frame->level.getCollision(yshape,Placement(pos));
					if (!c.IsCollision()) {
						ok = true;
						break;
					}
				}
			}
			if (!ok) {
				pos.y-=3;
				hurt = 20;
				// squashed y!
			}
		}
		// Movement collisions
		while ((newpos-pos).GetMagnitudeSquared()>0.01) {
			float old = pos.y;
			
			if (abs(pos.y-newpos.y)<1)
				pos.y = newpos.y;
			else if (pos.y<newpos.y) pos.y++;
			else if (pos.y>newpos.y) pos.y--;
			Collision c = frame->level.getCollision(yshape,Placement(pos),&groundtype);
			if (c.IsCollision()) {
				pos.y = old;
				newpos.y = old;
				if (delta.y>0)
					onground = true;
				delta.y = 0;
			}
			
			old = pos.x;
			if (abs(pos.x-newpos.x)<1)
				pos.x = newpos.x;
			else if (pos.x<newpos.x) pos.x++;
			else if (pos.x>newpos.x) pos.x--;
			
			c = frame->level.getCollision(xshape,Placement(pos));
			if (c.IsCollision()) {
				pos.x = old;
				newpos.x = old;
				onside = delta.x>0 ? 1 : -1;
				delta.x=0;
			}
			
		}
		while (true) {
			Collision c = frame->level.getCollision(yshape,Placement(pos));
			if (c.IsCollision()) {
				pos.y--;
			} else
				break;
		}
		xshape.MoveTo(pos);
		yshape.MoveTo(pos);
	} else 
		pos+=delta*timerate;
	
	Bitmap *bmp = frame->game->rm.load<OLBitmap>("data/particle.png")->b;
	Bitmap *bmpsmoke = frame->game->rm.load<OLBitmap>("data/smoke.png")->b;
	//Bitmap *bmp = frame->game->rm.load<OLBitmap>("data/ascii/ascii"+ToString(rand()%8+1)+".png")->b;
	
	// Input
	if (hurt<=0) {
		bool left = frame->game->joystick->getInput(VirtualJoystick::LEFT)->getValue();
		bool right = frame->game->joystick->getInput(VirtualJoystick::RIGHT)->getValue();
		bool jump = frame->game->joystick->getInput(VirtualJoystick::B)->getValue() ||
					frame->game->joystick->getInput(VirtualJoystick::UP)->getValue();
		if (jump && jumpcount==0) {
			if (onground) {
				delta.y = -7.0;
				for (int i=0; i<5; i++) {
					float angle = toRadians(rand()%180);
					float speed = (rand()%100)*0.02;
					Particle *p = new Particle(pos,Vec2D(speed*cos(angle),speed*sin(angle)),200,bmp);
					p->angle = toRadians(rand()%360);
					p->scale = speed/2;
					frame->particles.add(p);
				}
				jumpcount++;
			} else if ((onside<0 && left) || (onside>0 && right)) {
				delta.y = -6.0;
				delta.x = -5.0 * onside;
				for (int i=0; i<3; i++) {
					float angle = toRadians(rand()%180 - 90*onside);
					float speed = (rand()%100)*0.02;
					Particle *p = new Particle(pos,Vec2D(speed*cos(angle),speed*sin(angle)),200,bmp);
					p->angle = toRadians(rand()%360);
					p->scale = speed/2;
					frame->particles.add(p);
				}
				jumpcount++;
			}
		} else if (!jump)
			jumpcount = 0;
		if (frame->game->joystick->getInput(VirtualJoystick::A)->getValue()) {
			charge+=timerate;
			
			rate += min(charge*charge*0.00001,3.0)*timerate;
			if (rate>1) {
				rate--;
				Vec2D dir(cos(angle),sin(angle));
				Vec2D start(pos.x+(flip?-armpos.x:armpos.x),pos.y+armpos.y);
				start += dir*40;
				
				float randangle = toRadians(rand()%360);
				dir = Vec2D(cos(randangle),sin(randangle));
				
				frame->particles.add(new Particle(start+dir*60, -2*dir, 30, ascii.getRandom()));
			}
	
		} else {
			if (charge>0) {
				rate = 80*min(charge*charge*0.00001,3.0);
				Vec2D dir(cos(angle),sin(angle));
				Vec2D start(pos.x+(flip?-armpos.x:armpos.x),pos.y+armpos.y);
				start += dir*40;
				for (int i=0; i<rate; i++)
					frame->shots.add(new Particle(start, (20.0*(i+1)/max(1.0f,rate))*dir, 300, bmp));
				
			}
			charge = 0;
		}
		float speed = 5;
		if (left && !right)
			speed = -speed;
		else if (!left && !right)
			speed = 0;
		delta.x +=(speed-delta.x)/((onground?5:20))*timerate;
		if (onground)
			anim = speed==0 ? &stand : &run;
		else
			anim = &this->jump;
		
	} else {
		anim = &hurting;
	}
	
	int mousex = ::mouse_x;
	int mousey = ::mouse_y;
	
	anim->update(timerate);
		
	float armoffset = cos(anim->getFrame() * 4 * PI / anim->size());
    angle = atan2(mousey-pos.y - armpos.y - armoffset,mousex-pos.x-armpos.x);
    flip = mousex<pos.x;
}
void Player::draw() {
	if (hurt<=0 && frame->game->joystick->getInput(VirtualJoystick::A)->getValue()) {
		Vec2D start(pos.x+(flip?-armpos.x:armpos.x),pos.y+armpos.y);
		float coss = cos(angle);
		float sinn = sin(angle);
		Vec2D end;
		
		float dist = 1000;
		while (true) {
			end.x = start.x+coss*dist;
			end.y = start.y+sinn*dist;
			Poly p;
			p.Add(start);
			p.Add(end);
			Collision c = frame->level.getCollision(p,Placement());
			if (!c.IsCollision())
				break;
			dist-=1.0;
			//	end = c.GetPoint();
		}
		
		start.x += cos(angle)*40;
		start.y += sin(angle)*40;
		for (int i=0; i<3; i+=3)
			Line(start,end,i).Draw(Rgba(1.0,1.0,1.0,0.2));
	}
	float armoffset = cos((int)anim->getFrame() * 4 * PI / anim->size());
	if (flip) {
		if (hurt<=0)
			arm->BlitRotated(pos.x - armpos.x, pos.y + armpos.y + armoffset, 10, 13, angle, Flipped( VERTICAL ) );
		anim->get()->Blit(pos.x - offset.x, pos.y - offset.y, Flipped( HORIZONTAL ));
	} else {
		if (hurt<=0)
			arm->BlitRotated(pos.x + armpos.x, pos.y + armpos.y + armoffset, 10, 13, angle );
		anim->get()->Blit(pos.x - offset.x, pos.y - offset.y);
	}
}
void Player::damage(float amount) {
	if (hurt>0) return;
	health -= amount;
	if (health<0) health = 0;
	hurt = 30;
	delta.y -= 3;
}
